@@ -1,24 +1,17 @@
 //!
-//! # A disk-storage replacement for the pure in-memory BTreeMap
-//!
-//! This module is non-invasive to external code except the `new` method.
+//! A disk-storage replacement for the pure in-memory BTreeMap.
 //!
 
 #[cfg(test)]
 mod test;
 
 use crate::{
-    basic::mapx_oc::{self, MapxOC, MapxOCIter},
+    basic::mapx_oc::{Entry, MapxOC, MapxOCIter, ValueMut},
     common::{InstanceCfg, SimpleVisitor},
 };
 use ruc::*;
 use serde::{de::DeserializeOwned, Serialize};
-use std::{
-    fmt,
-    marker::PhantomData,
-    mem::ManuallyDrop,
-    ops::{Deref, DerefMut},
-};
+use std::{fmt, marker::PhantomData};
 
 /// To solve the problem of unlimited memory usage,
 /// use this to replace the original in-memory `BTreeMap<_, _>`.
@@ -99,11 +92,11 @@ where
 
     /// Imitate the behavior of 'BTreeMap<_>.get_mut(...)'
     #[inline(always)]
-    pub fn get_mut(&mut self, key: &K) -> Option<ValueMut<'_, K, V>> {
+    pub fn get_mut(&mut self, key: &K) -> Option<ValueMut<'_, Vec<u8>, V>> {
         let k = convert!(key);
         self.inner
             .get(&k)
-            .map(move |v| ValueMut::new(self, key.clone(), v))
+            .map(move |v| ValueMut::new(&mut self.inner, k, v))
     }
 
     /// Imitate the behavior of 'BTreeMap<_>.len()'.
@@ -120,21 +113,21 @@ where
 
     /// Imitate the behavior of 'BTreeMap<_>.insert(...)'.
     #[inline(always)]
-    pub fn insert(&mut self, key: K, value: V) -> Option<V> {
+    pub fn insert(&mut self, key: K, value: &V) -> Option<V> {
         let key = convert!(&key);
         self.inner.insert(key, value)
     }
 
     /// Similar with `insert`, but ignore the old value.
     #[inline(always)]
-    pub fn set_value(&mut self, key: K, value: V) {
+    pub fn set_value(&mut self, key: K, value: &V) {
         let key = convert!(&key);
         self.inner.set_value(key, value);
     }
 
     /// Imitate the behavior of '.entry(...).or_insert(...)'
     #[inline(always)]
-    pub fn entry(&mut self, key: K) -> mapx_oc::Entry<'_, Vec<u8>, V> {
+    pub fn entry(&mut self, key: K) -> Entry<'_, Vec<u8>, V> {
         let key = convert!(&key);
         self.inner.entry(key)
     }
@@ -180,115 +173,6 @@ where
 // End of the self-implementation for Mapx //
 /////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////////////////////////////
-// Begin of the implementation of ValueMut(returned by `self.get_mut`) for Mapx //
-/********************************************************************************/
-
-/// Returned by `<Mapx>.get_mut(...)`
-#[derive(Debug)]
-pub struct ValueMut<'a, K, V>
-where
-    K: Clone
-        + PartialEq
-        + Eq
-        + PartialOrd
-        + Ord
-        + Serialize
-        + DeserializeOwned
-        + fmt::Debug,
-    V: Serialize + DeserializeOwned + fmt::Debug,
-{
-    hdr: &'a mut Mapx<K, V>,
-    key: ManuallyDrop<K>,
-    value: ManuallyDrop<V>,
-}
-
-impl<'a, K, V> ValueMut<'a, K, V>
-where
-    K: Clone
-        + PartialEq
-        + Eq
-        + PartialOrd
-        + Ord
-        + Serialize
-        + DeserializeOwned
-        + fmt::Debug,
-    V: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn new(hdr: &'a mut Mapx<K, V>, key: K, value: V) -> Self {
-        ValueMut {
-            hdr,
-            key: ManuallyDrop::new(key),
-            value: ManuallyDrop::new(value),
-        }
-    }
-}
-
-/// NOTE: Very Important !!!
-impl<'a, K, V> Drop for ValueMut<'a, K, V>
-where
-    K: Clone
-        + PartialEq
-        + Eq
-        + PartialOrd
-        + Ord
-        + Serialize
-        + DeserializeOwned
-        + fmt::Debug,
-    V: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn drop(&mut self) {
-        // This operation is safe within a `drop()`.
-        // SEE: [**ManuallyDrop::take**](std::mem::ManuallyDrop::take)
-        unsafe {
-            self.hdr.set_value(
-                ManuallyDrop::take(&mut self.key),
-                ManuallyDrop::take(&mut self.value),
-            );
-        };
-    }
-}
-
-impl<'a, K, V> Deref for ValueMut<'a, K, V>
-where
-    K: Clone
-        + PartialEq
-        + Eq
-        + PartialOrd
-        + Ord
-        + Serialize
-        + DeserializeOwned
-        + fmt::Debug,
-    V: Serialize + DeserializeOwned + fmt::Debug,
-{
-    type Target = V;
-
-    fn deref(&self) -> &Self::Target {
-        &self.value
-    }
-}
-
-impl<'a, K, V> DerefMut for ValueMut<'a, K, V>
-where
-    K: Clone
-        + PartialEq
-        + Eq
-        + PartialOrd
-        + Ord
-        + Serialize
-        + DeserializeOwned
-        + fmt::Debug,
-    V: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.value
-    }
-}
-
-/******************************************************************************/
-// End of the implementation of ValueMut(returned by `self.get_mut`) for Mapx //
-////////////////////////////////////////////////////////////////////////////////
-
 //////////////////////////////////////////////////
 // Begin of the implementation of Iter for Mapx //
 /************************************************/
