@@ -27,14 +27,14 @@ fn basic_cases() {
         });
 
         (0..cnt).map(|i| (i, gen_sample(i))).for_each(|(i, b)| {
-            hdr_i.entry(i).or_insert(b.clone());
+            hdr_i.entry(i).or_insert(&b);
             assert_eq!(1 + i as usize, hdr_i.len());
             assert_eq!(pnk!(hdr_i.get(&i)).idx, i);
             assert_eq!(hdr_i.remove(&i), Some(b.clone()));
             assert_eq!(i as usize, hdr_i.len());
             assert!(hdr_i.get(&i).is_none());
-            assert!(hdr_i.insert(i, b.clone()).is_none());
-            assert!(hdr_i.insert(i, b).is_some());
+            assert!(hdr_i.insert(i, &b).is_none());
+            assert!(hdr_i.insert(i, &b).is_some());
         });
 
         assert_eq!(cnt, hdr_i.len());
