@@ -1,7 +1,3 @@
-//!
-//! # Disk Storage Implementation
-//!
-
 use crate::{
     basic::mapx_raw::{MapxRaw, MapxRawIter},
     common::InstanceCfg,
@@ -9,7 +5,6 @@ use crate::{
 };
 use ruc::*;
 use serde::{de::DeserializeOwned, Serialize};
-use sled::IVec;
 use std::{
     fmt,
     marker::PhantomData,
@@ -53,7 +48,7 @@ where
         Self {
             prefix: cfg.prefix,
             item_cnt: cfg.item_cnt,
-            data_set_idx: cfg.data_set_idx,
+            area_idx: cfg.area_idx,
         }
     }
 }
@@ -135,16 +130,37 @@ where
     }
 
     #[inline(always)]
-    pub(super) fn insert(&mut self, key: K, value: V) -> Option<V> {
+    pub(super) fn insert(&mut self, key: K, value: &V) -> Option<V> {
         self.set_value(key, value)
             .map(|v| pnk!(bcs::from_bytes(&v)))
     }
 
-    // Similar with `insert`, but ignore if the old value is exist.
     #[inline(always)]
-    pub(super) fn set_value(&mut self, key: K, value: V) -> Option<IVec> {
+    pub(super) fn _insert(&mut self, key: &[u8], value: &V) -> Option<V> {
+        self._set_value(key, value)
+            .map(|v| pnk!(bcs::from_bytes(&v)))
+    }
+
+    #[inline(always)]
+    pub(super) fn __insert(&mut self, key: &[u8], value: &[u8]) -> Option<V> {
+        self.__set_value(key, value)
+            .map(|v| pnk!(bcs::from_bytes(&v)))
+    }
+
+    #[inline(always)]
+    pub(super) fn set_value(&mut self, key: K, value: &V) -> Option<Vec<u8>> {
         self.inner
-            .insert(&key.into_bytes(), &pnk!(bcs::to_bytes(&value)))
+            .insert(&key.into_bytes(), &pnk!(bcs::to_bytes(value)))
+    }
+
+    #[inline(always)]
+    pub(super) fn _set_value(&mut self, key: &[u8], value: &V) -> Option<Vec<u8>> {
+        self.inner.insert(key, &pnk!(bcs::to_bytes(value)))
+    }
+
+    #[inline(always)]
+    pub(super) fn __set_value(&mut self, key: &[u8], value: &[u8]) -> Option<Vec<u8>> {
+        self.inner.insert(key, value)
     }
 
     #[inline(always)]
@@ -213,12 +229,12 @@ where
     }
 
     #[inline(always)]
-    pub(super) fn unset_value(&mut self, key: &K) -> Option<IVec> {
+    pub(super) fn unset_value(&mut self, key: &K) -> Option<Vec<u8>> {
         self.inner.remove(&key.to_bytes())
     }
 
     #[inline(always)]
-    pub(super) fn _unset_value(&mut self, key: &[u8]) -> Option<IVec> {
+    pub(super) fn _unset_value(&mut self, key: &[u8]) -> Option<Vec<u8>> {
         self.inner.remove(key)
     }
 
