@@ -1,17 +1,15 @@
 //!
-//! # A disk-storage replacement for the pure in-memory BTreeMap
-//!
-//! This module is non-invasive to external code except the `new` method.
+//! A disk-storage replacement for the pure in-memory BTreeMap.
 //!
 
 mod backend;
+
 #[cfg(test)]
 mod test;
 
 use crate::common::{InstanceCfg, SimpleVisitor};
 use ruc::*;
 use serde::{de::DeserializeOwned, Serialize};
-use sled::IVec;
 use std::{
     fmt,
     mem::{size_of, transmute, ManuallyDrop},
@@ -139,16 +137,40 @@ where
 
     /// Imitate the behavior of 'BTreeMap<_>.insert(...)'.
     #[inline(always)]
-    pub fn insert(&mut self, key: K, value: V) -> Option<V> {
+    pub fn insert(&mut self, key: K, value: &V) -> Option<V> {
         self.inner.insert(key, value)
     }
 
+    /// same as the funtion without the '_' prefix, but use bytes key
+    #[inline(always)]
+    pub fn _insert(&mut self, key: &[u8], value: &V) -> Option<V> {
+        self.inner._insert(key, value)
+    }
+
+    /// same as the funtion without the '_' prefix, but use bytes key&value
+    #[inline(always)]
+    pub fn __insert(&mut self, key: &[u8], value: &[u8]) -> Option<V> {
+        self.inner.__insert(key, value)
+    }
+
     /// Similar with `insert`, but ignore the old value.
     #[inline(always)]
-    pub fn set_value(&mut self, key: K, value: V) {
+    pub fn set_value(&mut self, key: K, value: &V) {
         self.inner.set_value(key, value);
     }
 
+    /// same as the funtion without the '_' prefix, but use bytes key
+    #[inline(always)]
+    pub fn _set_value(&mut self, key: &[u8], value: &V) {
+        self.inner._set_value(key, value);
+    }
+
+    /// same as the funtion without the '_' prefix, but use bytes key&value
+    #[inline(always)]
+    pub fn __set_value(&mut self, key: &[u8], value: &[u8]) {
+        self.inner.__set_value(key, value);
+    }
+
     /// Imitate the behavior of '.entry(...).or_insert(...)'
     #[inline(always)]
     pub fn entry(&mut self, key: K) -> Entry<'_, K, V> {
@@ -251,7 +273,7 @@ where
     K: OrderConsistKey,
     V: Serialize + DeserializeOwned + fmt::Debug,
 {
-    fn new(hdr: &'a mut MapxOC<K, V>, key: K, value: V) -> Self {
+    pub(crate) fn new(hdr: &'a mut MapxOC<K, V>, key: K, value: V) -> Self {
         ValueMut {
             hdr,
             key: ManuallyDrop::new(key),
@@ -272,7 +294,7 @@ where
         unsafe {
             self.hdr.set_value(
                 ManuallyDrop::take(&mut self.key),
-                ManuallyDrop::take(&mut self.value),
+                &ManuallyDrop::take(&mut self.value),
             );
         };
     }
@@ -324,23 +346,12 @@ where
     V: 'a + fmt::Debug + Serialize + DeserializeOwned,
 {
     /// Imitate the `btree_map/btree_map::Entry.or_insert(...)`.
-    pub fn or_insert(self, default: V) -> ValueMut<'a, K, V> {
+    pub fn or_insert(self, default: &V) -> ValueMut<'a, K, V> {
         if !self.hdr.contains_key(&self.key) {
             self.hdr.set_value(self.key.clone(), default);
         }
         pnk!(self.hdr.get_mut(&self.key))
     }
-
-    /// Imitate the `btree_map/btree_map::Entry.or_insert_with(...)`.
-    pub fn or_insert_with<F>(self, default: F) -> ValueMut<'a, K, V>
-    where
-        F: FnOnce() -> V,
-    {
-        if !self.hdr.contains_key(&self.key) {
-            self.hdr.set_value(self.key.clone(), default());
-        }
-        pnk!(self.hdr.get_mut(&self.key))
-    }
 }
 
 /***********************************************/
@@ -446,18 +457,6 @@ pub trait OrderConsistKey:
     }
 }
 
-impl OrderConsistKey for IVec {
-    #[inline(always)]
-    fn to_bytes(&self) -> Vec<u8> {
-        self.to_vec()
-    }
-
-    #[inline(always)]
-    fn from_slice(b: &[u8]) -> Result<Self> {
-        Ok(IVec::from(b))
-    }
-}
-
 impl OrderConsistKey for Vec<u8> {
     #[inline(always)]
     fn to_bytes(&self) -> Vec<u8> {
