@@ -1,16 +1,12 @@
 //!
-//! # A disk-storage replacement for the pure in-memory BTreeMap
-//!
-//! This module is non-invasive to external code except the `new` method.
+//! A disk-storage replacement for the pure in-memory BTreeMap.
 //!
 
-mod backend;
 #[cfg(test)]
 mod test;
 
-use crate::common::{InstanceCfg, SimpleVisitor, VSDB};
+use crate::common::{engines, InstanceCfg, SimpleVisitor};
 use ruc::*;
-use sled::IVec;
 use std::{
     mem::ManuallyDrop,
     ops::{Deref, DerefMut, RangeBounds},
@@ -20,13 +16,13 @@ use std::{
 /// use this to replace the original in-memory `BTreeMap<_, _>`.
 #[derive(PartialEq, Eq, Debug)]
 pub struct MapxRaw {
-    inner: backend::MapxRaw,
+    inner: engines::Mapx,
 }
 
 impl From<InstanceCfg> for MapxRaw {
     fn from(cfg: InstanceCfg) -> Self {
         Self {
-            inner: backend::MapxRaw::from(cfg),
+            inner: engines::Mapx::from(cfg),
         }
     }
 }
@@ -46,7 +42,7 @@ impl MapxRaw {
     #[inline(always)]
     pub fn new() -> Self {
         MapxRaw {
-            inner: backend::MapxRaw::must_new(VSDB.alloc_prefix()),
+            inner: engines::Mapx::new(),
         }
     }
 
@@ -57,20 +53,26 @@ impl MapxRaw {
 
     /// Imitate the behavior of 'BTreeMap<_>.get(...)'
     #[inline(always)]
-    pub fn get(&self, key: &[u8]) -> Option<IVec> {
-        self.inner.get(key)
+    pub fn get(&self, key: &[u8]) -> Option<Vec<u8>> {
+        self.inner.get(key).map(|iv| iv.to_vec())
+    }
+
+    /// Check if a key is exists.
+    #[inline(always)]
+    pub fn contains_key(&self, key: &[u8]) -> bool {
+        self.get(key).is_some()
     }
 
     /// less or equal value
     #[inline(always)]
-    pub fn get_le(&self, key: &[u8]) -> Option<(IVec, IVec)> {
-        self.inner.get_le(key)
+    pub fn get_le(&self, key: &[u8]) -> Option<(Vec<u8>, Vec<u8>)> {
+        self.range(..=key).next_back()
     }
 
     /// great or equal value
     #[inline(always)]
-    pub fn get_ge(&self, key: &[u8]) -> Option<(IVec, IVec)> {
-        self.inner.get_ge(key)
+    pub fn get_ge(&self, key: &[u8]) -> Option<(Vec<u8>, Vec<u8>)> {
+        self.range(key..).next()
     }
 
     /// Imitate the behavior of 'BTreeMap<_>.get_mut(...)'
@@ -78,7 +80,7 @@ impl MapxRaw {
     pub fn get_mut(&mut self, key: &[u8]) -> Option<ValueMut<'_>> {
         self.inner
             .get(key)
-            .map(move |v| ValueMut::new(self, IVec::from(key), v))
+            .map(move |v| ValueMut::new(self, key.to_owned(), v.to_vec()))
     }
 
     /// Imitate the behavior of 'BTreeMap<_>.len()'.
@@ -93,12 +95,6 @@ impl MapxRaw {
         self.inner.is_empty()
     }
 
-    /// Imitate the behavior of 'BTreeMap<_>.insert(...)'.
-    #[inline(always)]
-    pub fn insert(&mut self, key: &[u8], value: &[u8]) -> Option<IVec> {
-        self.inner.insert(key, value)
-    }
-
     /// Imitate the behavior of '.entry(...).or_insert(...)'
     #[inline(always)]
     pub fn entry<'a>(&'a mut self, key: &'a [u8]) -> Entry<'a> {
@@ -121,16 +117,16 @@ impl MapxRaw {
         }
     }
 
-    /// Check if a key is exists.
+    /// Imitate the behavior of 'BTreeMap<_>.insert(...)'.
     #[inline(always)]
-    pub fn contains_key(&self, key: &[u8]) -> bool {
-        self.inner.contains_key(key)
+    pub fn insert(&mut self, key: &[u8], value: &[u8]) -> Option<Vec<u8>> {
+        self.inner.insert(key, value).map(|iv| iv.to_vec())
     }
 
     /// Try to remove an entry
     #[inline(always)]
-    pub fn remove(&mut self, key: &[u8]) -> Option<IVec> {
-        self.inner.remove(key)
+    pub fn remove(&mut self, key: &[u8]) -> Option<Vec<u8>> {
+        self.inner.remove(key).map(|iv| iv.to_vec())
     }
 
     /// Clear all data.
@@ -152,12 +148,12 @@ impl MapxRaw {
 #[derive(PartialEq, Eq, Debug)]
 pub struct ValueMut<'a> {
     hdr: &'a mut MapxRaw,
-    key: ManuallyDrop<IVec>,
-    value: ManuallyDrop<IVec>,
+    key: ManuallyDrop<Vec<u8>>,
+    value: ManuallyDrop<Vec<u8>>,
 }
 
 impl<'a> ValueMut<'a> {
-    fn new(hdr: &'a mut MapxRaw, key: IVec, value: IVec) -> Self {
+    fn new(hdr: &'a mut MapxRaw, key: Vec<u8>, value: Vec<u8>) -> Self {
         ValueMut {
             hdr,
             key: ManuallyDrop::new(key),
@@ -181,7 +177,7 @@ impl<'a> Drop for ValueMut<'a> {
 }
 
 impl<'a> Deref for ValueMut<'a> {
-    type Target = IVec;
+    type Target = Vec<u8>;
 
     fn deref(&self) -> &Self::Target {
         &self.value
@@ -216,17 +212,6 @@ impl<'a> Entry<'a> {
         }
         pnk!(self.hdr.get_mut(self.key))
     }
-
-    /// Imitate the `btree_map/btree_map::Entry.or_insert_with(...)`.
-    pub fn or_insert_with<F>(self, default: F) -> ValueMut<'a>
-    where
-        F: FnOnce() -> Vec<u8>,
-    {
-        if !self.hdr.contains_key(self.key) {
-            self.hdr.insert(self.key, &default());
-        }
-        pnk!(self.hdr.get_mut(self.key))
-    }
 }
 
 /***********************************************/
@@ -239,19 +224,21 @@ impl<'a> Entry<'a> {
 
 /// Iter over [MapxRaw](self::MapxRaw).
 pub struct MapxRawIter {
-    iter: backend::MapxRawIter,
+    iter: engines::MapxIter,
 }
 
 impl Iterator for MapxRawIter {
-    type Item = (IVec, IVec);
+    type Item = (Vec<u8>, Vec<u8>);
     fn next(&mut self) -> Option<Self::Item> {
-        self.iter.next()
+        self.iter.next().map(|(ik, iv)| (ik.to_vec(), iv.to_vec()))
     }
 }
 
 impl DoubleEndedIterator for MapxRawIter {
     fn next_back(&mut self) -> Option<Self::Item> {
-        self.iter.next_back()
+        self.iter
+            .next_back()
+            .map(|(ik, iv)| (ik.to_vec(), iv.to_vec()))
     }
 }
 
