@@ -1,24 +1,17 @@
 //!
-//! # A disk-storage replacement for the pure in-memory Vec
-//!
-//! This module is non-invasive to external code except the `new` method.
+//! A disk-storage replacement for the pure in-memory Vec.
 //!
 
 #[cfg(test)]
 mod test;
 
 use crate::{
-    basic::mapx_oc::{MapxOC, MapxOCIter},
+    basic::mapx_oc::{MapxOC, MapxOCIter, ValueMut},
     common::{InstanceCfg, SimpleVisitor},
 };
 use ruc::*;
 use serde::{de::DeserializeOwned, Serialize};
-use std::{
-    cmp::Ordering,
-    fmt,
-    mem::ManuallyDrop,
-    ops::{Deref, DerefMut},
-};
+use std::{cmp::Ordering, fmt};
 
 /// To solve the problem of unlimited memory usage,
 /// use this to replace the original in-memory 'Vec'.
@@ -81,10 +74,10 @@ where
 
     /// Imitate the behavior of 'Vec<_>.get_mut(...)'
     #[inline(always)]
-    pub fn get_mut(&mut self, idx: usize) -> Option<ValueMut<'_, T>> {
+    pub fn get_mut(&mut self, idx: usize) -> Option<ValueMut<'_, usize, T>> {
         self.inner
             .get(&idx)
-            .map(move |v| ValueMut::new(self, idx, v))
+            .map(move |v| ValueMut::new(&mut self.inner, idx, v))
     }
 
     /// Imitate the behavior of 'Vec<_>.last()'
@@ -109,17 +102,17 @@ where
 
     /// Imitate the behavior of 'Vec<_>.push(...)'
     #[inline(always)]
-    pub fn push(&mut self, b: T) {
+    pub fn push(&mut self, b: &T) {
         self.inner.insert(self.len(), b);
     }
 
     /// Imitate the behavior of 'Vec<_>.insert()'
     #[inline(always)]
-    pub fn insert(&mut self, idx: usize, v: T) {
+    pub fn insert(&mut self, idx: usize, v: &T) {
         match self.len().cmp(&idx) {
             Ordering::Greater => {
                 self.inner.range(idx..self.len()).for_each(|(i, v)| {
-                    self.inner.insert(i + 1, v);
+                    self.inner.insert(i + 1, &v);
                 });
                 self.inner.insert(idx, v);
             }
@@ -146,7 +139,7 @@ where
             let last_idx = self.len() - 1;
             let ret = self.inner.remove(&idx).unwrap();
             self.inner.range((1 + idx)..).for_each(|(i, v)| {
-                self.inner.insert(i - 1, v);
+                self.inner.insert(i - 1, &v);
             });
             self.inner.remove(&last_idx);
             return ret;
@@ -161,7 +154,7 @@ where
             let last_idx = self.len() - 1;
             let ret = self.inner.remove(&idx).unwrap();
             if let Some(v) = self.inner.remove(&last_idx) {
-                self.inner.insert(idx, v);
+                self.inner.insert(idx, &v);
             }
             return ret;
         }
@@ -170,7 +163,7 @@ where
 
     /// Imitate the behavior of 'Vec<_>.update(idx, value)'
     #[inline(always)]
-    pub fn update(&mut self, idx: usize, b: T) -> Option<T> {
+    pub fn update(&mut self, idx: usize, b: &T) -> Option<T> {
         if idx < self.len() {
             return self.inner.insert(idx, b);
         }
@@ -196,74 +189,6 @@ where
 // End of the self-implementation for Vecx //
 /////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////////////////////////////
-// Begin of the implementation of ValueMut(returned by `self.get_mut`) for Vecx //
-/********************************************************************************/
-
-/// Returned by `<Vecx>.get_mut(...)`
-#[derive(Debug)]
-pub struct ValueMut<'a, T>
-where
-    T: Serialize + DeserializeOwned + fmt::Debug,
-{
-    hdr: &'a mut Vecx<T>,
-    idx: usize,
-    value: ManuallyDrop<T>,
-}
-
-impl<'a, T> ValueMut<'a, T>
-where
-    T: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn new(hdr: &'a mut Vecx<T>, idx: usize, value: T) -> Self {
-        ValueMut {
-            hdr,
-            idx,
-            value: ManuallyDrop::new(value),
-        }
-    }
-}
-
-/// NOTE: Very Important !!!
-impl<'a, T> Drop for ValueMut<'a, T>
-where
-    T: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn drop(&mut self) {
-        // This operation is safe within a `drop()`.
-        // SEE: [**ManuallyDrop::take**](std::mem::ManuallyDrop::take)
-        unsafe {
-            self.hdr
-                .update(self.idx, ManuallyDrop::take(&mut self.value))
-                .unwrap();
-        };
-    }
-}
-
-impl<'a, T> Deref for ValueMut<'a, T>
-where
-    T: Serialize + DeserializeOwned + fmt::Debug,
-{
-    type Target = T;
-
-    fn deref(&self) -> &Self::Target {
-        &self.value
-    }
-}
-
-impl<'a, T> DerefMut for ValueMut<'a, T>
-where
-    T: Serialize + DeserializeOwned + fmt::Debug,
-{
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.value
-    }
-}
-
-/******************************************************************************/
-// End of the implementation of ValueMut(returned by `self.get_mut`) for Vecx //
-////////////////////////////////////////////////////////////////////////////////
-
 //////////////////////////////////////////////////
 // Begin of the implementation of Iter for Vecx //
 /************************************************/
